import 'package:cl_appbase/api.dart';
import 'package:cl_appbase/src/api/api_error.dart';
import 'package:cl_appbase/src/api/api_resource.dart';
import 'package:cl_appbase/src/api/utils.dart';

import 'api.dart';

class ApiBase {
  final String baseUrl;
  final List<ApiResource> _resources = [];

  ApiBase(this.baseUrl);

  void bindResource(ApiResource resource) {
    if (_resources.contains(resource)) {
      throw ApiException('Cannot bind: Resource already bound to this api.');
    }

    if (resource.api != this) {
      throw ApiError('Cannot bind: Resource belongs to different api.');
    }

    _resources.add(resource);
  }

  Map<String, String> getRequestHeaders() {
    //TODO auth token and stuff
    return {};
  }

  Future<ApiResponse<TData>> getObject<TData>(String postfix,
      {Map<String, dynamic>? urlParams, DTOFactory? factory}) async {
    return await request<TData, TData>(
        baseUrl, postfix, urlParams, ApiRequestMethod.GET,
        headers: getRequestHeaders(), factory: factory);
  }

  Future<ApiResponse<List<TData>>> getList<TData>(String postfix,
      {Map<String, dynamic>? urlParams, DTOFactory? factory}) async {
    return await request<TData, List<TData>>(
        baseUrl, postfix, urlParams, ApiRequestMethod.GET,
        headers: getRequestHeaders(), factory: factory);
  }

  Future<ApiResponse<TData>> postObject<TData>(String postfix, TData object,
      {Map<String, dynamic>? urlParams, DTOFactory? factory}) async {
    return await request<TData, TData>(
        baseUrl, postfix, urlParams, ApiRequestMethod.POST,
        headers: getRequestHeaders(), factory: factory, body: object);
  }

  Future<ApiResponse<TData>> patchObject<TData>(String postfix, TData object,
      {Map<String, dynamic>? urlParams, DTOFactory? factory}) async {
    return await request<TData, TData>(
        baseUrl, postfix, urlParams, ApiRequestMethod.PATCH,
        headers: getRequestHeaders(), factory: factory, body: object);
  }
}
