import 'package:cl_appbase/api.dart';
import 'package:cl_appbase/src/api/utils.dart';

class ApiResource<TData extends TransferObject> {
  final String endpointPostfix;
  final DTOFactory<TData> factory;
  final ApiBase api;

  const ApiResource(this.api, this.endpointPostfix, this.factory);

  Future<List<TData>> getList(int offset, int limit,
      [Map<String, dynamic>? params]) async {
    final response = await api.getList<TData>(endpointPostfix,
        urlParams: params, factory: factory);
    response.throwOnError();
    return response.data;
  }

  Future<TData> getElement(int id, [Map<String, dynamic>? params]) async {
    final response = await api.getObject('$endpointPostfix/$id',
        urlParams: params, factory: factory);
    response.throwOnError();
    return response.data;
  }

  Future<TData> postElement(TData element,
      [Map<String, dynamic>? params]) async {
    final response = await api.postObject(endpointPostfix, element,
        urlParams: params, factory: factory);
    response.throwOnError();
    return response.data;
  }

  Future<TData> patchElement(TData element,
      [Map<String, dynamic>? params]) async {
    final response = await api.patchObject(endpointPostfix, element,
        urlParams: params, factory: factory);
    response.throwOnError();
    return response.data;
  }
}
